#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	ComponentReplacement
#      By:	sdmitch
#   Usage:	Randomly replace the "Target" components with a "Source" component.
#	 	 Note:	All "Target" components MUST have the same name or prefix and all "Source" components MUST 
#						have the same name or prefix.  All "Target" components are randomly replaced by a "Source".
#						If some are selected, then only those are replaced.  The "Target" component can be anything
#						and can all be copies of a single component.  THE COMPONENT NAME, NOT THE COMPONENT
#						DEFINITION NAME IS USED FOR IDENTIFICATION OF THE "Source" AND "Target" COMPONENTS.  You
#						can access the component name through Entity Info or the Ruby Console by first selecting
#						the Source or Target components and using the appropriate code line below:
#						Sketchup.active_model.selection.each{|c| c.name='Source'}
#						Sketchup.active_model.selection.each{|c| c.name='Target'}
#    Date:	Jun 2011
#------------------------------------------------------------------------------------------------

require 'Sketchup'

module SDM_RCR

	def self.replace_targets
		mod=Sketchup.active_model
		ent=mod.entities
		sel=mod.selection
		@rand_replace = "Yes" if !@rand_replace;@source_name = "Source" if !@source_name;@target_name = "Target" if !@target_name
		prompts = ["Random Replacement?","Source Name/Prefix:","Target Name/Prefix:"]
		ans=UI.inputbox(prompts,[@rand_replace,@source_name,@target_name],["Yes|No"],"Component Replacement")
		return nil if !ans
		@rand_replace,@source_name,@target_name=ans
		if @source_name==@target_name
			UI.messagebox "The source and target names must be different"
			return nil
		end
		# get the source component definitions
		source_components=[]
		for e in ent
			if e.is_a? Sketchup::ComponentInstance
				if e.name[0...@source_name.length]==@source_name
					source_components.push e
				end
			end
		end
		#get the target components definitions
		target_components=[]; sel = ent if sel.empty?
		for e in sel
			if e.is_a? Sketchup::ComponentInstance
				if e.name[0...@target_name.length]==@target_name
					target_components.push e
				end
			end
		end
		
		# begin
			mod.start_operation "Component Replacement"
			@last=0
			for tc in target_components
				if @rand_replace == "Yes"
					begin
						i=rand(source_components.length)
					end until i != @last
					tc.definition=source_components[i].definition
					@last = i
				else
					tc.definition=source_components[@last].definition
					@last += 1; @last = 0 if @last == source_components.length
				end
			end
			mod.commit_operation
		# end until UI.messagebox("Do Random Replacement again?",MB_YESNO)==7
	end
	
end

if !file_loaded?(File.basename(__FILE__))
	UI.menu("plugins").add_item("Component Replacement") { SDM_RCR.replace_targets }
	file_loaded(File.basename(__FILE__))
end

